from AccessControl import getSecurityManager
from Globals import MessageDialog
from DateTime import DateTime

from os.path import join as pathJoin
from Config import reserved_names, error_messages
from sys import modules

from ZopeSyncer import ZopeSyncer

class Syncer:    
    def _isSyncable(self, m):
       if m in self.getSyncable():
            return 1

    def _isReserved(self, p):
        if len(p) == 2 and p[1] in reserved_names:
            return 1

    def _compare_lists(self, s_dict, d_dict):
        ''' compare the lists from the two zopes '''
        _l = []
        d_keys = d_dict.keys()
        s_keys = s_dict.keys()

        for item in s_dict.keys():
            # ok its not the dest, hence its missing
            if s_dict[item]['id'] not in d_keys: 
                s_dict[item]['status'] = 'missing'
                _l.append(s_dict[item])
            # ok its on the dest, but the last modified time is earlier
            elif s_dict[item]['last_modified_time'] > d_dict[item]['last_modified_time']:
                s_dict[item]['status'] = 'out of date'
                _l.append(s_dict[item])
            # not sure what else to check
            else: 
                s_dict[item]['status'] = 'ok'
                _l.append(s_dict[item])
        # ok check its on the destination, but not on the original
        for item in d_dict.keys():
            if d_dict[item]['id'] not in s_keys: 
                d_dict[item]['status'] = 'extra'
                _l.append(d_dict[item])
        return _l

    def _get_user(self):
        """ Get the user information """
        # specified the user
        if self.override:
            return { 'user':self.user, 'pwd':self.pwd }
        else:
            # figure out who the current user is
            # and return that
            _user = getSecurityManager().getUser()
            return { 'user':_user.getId(), 'pwd':_user._getPassword() }

    def _get_time(self):
        # get time for logging
        # could be done using DateTime, but i think I want to fiddle this
        return str(DateTime())

    def _log(self, msgs):
        # log to ZSyncer
        # this will become more configurable
        file = open(self.logfile, 'a')

        # write out the log
        file.write('%s\tStarted\n' % self._get_time())
        for m in msgs: file.write('%s\t%s\n' % (self._get_time(), m))
        file.write('%s\tFinished\n' % self._get_time())

        file.close()

    def _get_dest(self, **args):
        """ gets the destination for calling methods on it """
        if not hasattr(self, 'dest_type'): self.dest_type = 'ZopeSyncer'
        k = self.dest_type
        imp = 'from Products.ZSyncer import %s as mod' % k
        exec(imp)
        func = getattr(mod, k)
        return func(**args)

    def _msg(self, msgs, REQUEST=None):
        # what to do with these messages
        ms = []
        for m in msgs: 
            # hack-o-rama, if i have a number, its a status code, so lets look it up and get a nicer message
            if type(m) == type(1): ms.append('%s: %s' % (m, error_messages.get(str(m), 'Unknown error occured')))
            elif m[:6] == 'Object': ms.append('%s' % m)
            else: ms.append(m)

        if self.log or self.approval: self._log(ms)
        return ms

